//***********************************************************************************//
//*	Project:	PSoC4 BLE Data Logging using the On-Board FRam						*//
//*																					*//
//*	File:		FRam.h			 													*//
//*	Version:	1.0																	*//
//*																					*//
//*	Author:		Bob Marlowe															*//
//*	(C) 2015	Jörg Meier Software - Entwicklung briefe@jmeiersoftware.de			*//
//***********************************************************************************//
//***********************************************************************************//
//*																					*//
//*		Principles of Operation														*//
//*																					*//
//*		Logging of data is done by maintaining a FIFO memory for datasets			*//
//*		of varying informations and varying sizes.									*//
//*		To handle the FIFO correctly some additional information are kept within	*//
//*		the FRam to continue logging after a power-down								*//
//*																					*//
//*		Logged records are maintained in form of a combination of a linked-list		*//
//*		and a circular buffer. This allows for inserting new records and retrieving	*//
//*		from beginning without having to move data down from the top when a record	*//
//*		is removed from the begin of the logging-area in the FRam					*//
//*																					*//
//*		The FRam is not used completely for logging, a part at the begin may be 	*//
//*		spared for other uses														*//
//*																					*//
//*																					*//
//***********************************************************************************//
//***********************************************************************************//
//**************** Libraries ********************************************************//
//***********************************************************************************//
#include <FRam.h>
#include <DataLogger.h>
#include <TimeKeeper.h>
#include <stdlib.h>
//***********************************************************************************//
//**************** Definitions and Macros *******************************************//
//***********************************************************************************//
#define TagSize		sizeof(struct LogTagging)
#define	LogSize		sizeof(struct DataLog)

//***********************************************************************************//
//******************* Structs, Typedefs and Enums ***********************************//
//***********************************************************************************//
struct	DataLog 												//	Needed for maintaining the data log
		{
			uint32	MagicNumber;								//	Indicator for a valid initialized log
			uint16	EntryCount;									//	Number of records actually stored
			void *	ActualPointer;								//	Address of first log entry
			void *	PrevPointer;								//	Address of previously written log entry
			void *	FreePointer;								//	Address of next free memory area
			void *	NewestPointer;								//	Address of last written log entry
			uint32	FreeSpace;									//	Amount of free memory in FRam
		};
typedef struct DataLog * DataLog;								//	Pointer to above structure		

//***********************************************************************************//
//******************* Data Area *****************************************************//
//***********************************************************************************//
static DataLog Log;												//	Maintainance data in SRam
TimeDate CurrentTime;		
static LogTagging CurrentTag = NULL;							//	Tag of the current (oldest) entry		
		
//***********************************************************************************//
//****************** Prototypes *****************************************************//
//***********************************************************************************//
void WriteDataLog(DataLog Log);									//	Write DataLog structure to FRam
void ReadCurrentTag(LogTagging Tag);							//	Real oldest tag into CurrentTag

//***********************************************************************************//
//****************** Program Area ***************************************************//
//***********************************************************************************//
void DataLogger_Start(void)										//	Initialize and start the DataLogger
{
	TimeKeeper_Start();											//	Start the timekeeper
	Log = malloc(LogSize);										//	Allocate memory
	CurrentTag = malloc(TagSize);	
	CurrentTime = TimeKeeper_Start();
	FRam_Start();
	FRam_Read((uint8*)Log,(void *)FRamSpare,LogSize);			//	Read Log Data
	if(Log->MagicNumber == MAGIC_NUMBER) return;				//	FRam data is OK, go on working
	DataLogger_ClearLog();										//	Initialize the Log data
}
//***********************************************************************************//

void DataLogger_Stop(void)										//	Halt logging service, keep data
{
	TimeKeeper_Stop();
	FRam_Stop();	
	free(Log);													//	Free allocated memory
	free(CurrentTag);
}
//***********************************************************************************//

uint32 DataLogger_ClearLog(void)								//	Clear the log, return free space in bytes
{
	Log->ActualPointer 	= NULL;									//	Initialization of Log;
	Log->PrevPointer	= NULL;
	Log->NewestPointer	= NULL;
	Log->MagicNumber	= MAGIC_NUMBER;
	Log->EntryCount	= 0;
	Log->FreePointer	= (void *)FRamSpare + LogSize;
	Log->FreeSpace		= FRamSize - (uint32)Log->FreePointer;
	WriteDataLog(Log);
	return Log->FreeSpace;
}
//***********************************************************************************//

uint32 DataLogger_LogEntry(uint16 EntryType,void * Data,uint16 Count)	// Store a log-record
{
LogTagging LogTag;													//	Prepended data to user-entry
uint32	SpaceNeeded = Count +  TagSize;								//	Amount of SRam for new entry
uint8 * NewLogEntry;												//	Pointer to the new entry data in SRam
uint32	FRamRetValue	= CYRET_SUCCESS;							//	Return value from FRam access
LogTagging PriorSRamTagPointer;										//	Pointer in SRam to prior entry

	if(SpaceNeeded > Log->FreeSpace) return Log->FreeSpace;			//	Don't do anything when not enough space left
	if((uint32)(Log->FreePointer + SpaceNeeded) >= FRamSize)
	{
		Log->FreePointer = (void *)FRamSpare + LogSize;				//	Roll over
	}
	NewLogEntry = malloc(SpaceNeeded);								//	Get memory to build a complete record
	LogTag = (LogTagging)NewLogEntry;								//	Just to address the Tag

//***** Setting up the prepended info LogTagging ****//	
	LogTag->EntryType	= EntryType;								//	Fill entry with log data
	LogTag->NextPointer	= NULL;										//	No next enty, this is the last
	LogTag->EntrySize	= Count;									//	User supplied record-size
	TimeKeeper_GetTimeAndDate(&(LogTag->LogTime));					//	Fill time & date
//***************************************************//
	
//*** Setting up the DataLog informations in SRam ***//	
	Log->FreeSpace -= SpaceNeeded;									//	Update DataLog informations
	Log->EntryCount++;
	if(Log->ActualPointer == NULL)	Log->ActualPointer	= Log->FreePointer;		//	Remember the first Entry
	Log->PrevPointer	= Log->NewestPointer;						//	Remember the previous
	Log->NewestPointer	= Log->FreePointer;							//	New entry will go here
//***************************************************//	
	
//********* Setup and write entry to FRam ***********//
	memmove(NewLogEntry + TagSize,Data,Count);						//	Fill entry with userdata
	FRamRetValue |= FRam_Write(NewLogEntry, Log->FreePointer,SpaceNeeded);//	... and write it to FRam
	free(NewLogEntry);												//	Release memory
	
	Log->FreePointer += SpaceNeeded;								//	Next free FRam address
//***************************************************//
	
//** Update the value of NextPointer in prev entry **//
	if(Log->PrevPointer)
	{
		PriorSRamTagPointer = malloc(TagSize);						//	Get memory for the prior entry
		FRamRetValue |= FRam_Read(PriorSRamTagPointer, Log->PrevPointer, TagSize);		//	Read log data for prior entry
		PriorSRamTagPointer->NextPointer = Log->NewestPointer;		//	Set NextPointer in SRam
		FRamRetValue |= FRam_Write(PriorSRamTagPointer,Log->PrevPointer, TagSize);		//	Write back updated data
		free(PriorSRamTagPointer);									//	Release memory
	}
//***************************************************//
	WriteDataLog(Log);												//	Write back the Log to FRam
	while(FRamRetValue);											//	Halt on error
	return Log->FreeSpace;											//	Return free space in FRam
}
//***********************************************************************************//

void * DataLogger_GetLogEntry(uint16*EntryType,uint16 * EntrySize)	//	Read oldest logged record, allocate and fill buffer
{
void * Entry;
uint32 SpaceNeeded;													//	Needed memory for the new entry
LogTagging Tag = malloc(TagSize);						
	if(Log->ActualPointer == NULL) return NULL;
	if(Tag == NULL)
	{
		return NULL;
	}
	FRam_Read((void *)Tag,Log->ActualPointer,TagSize);				//	Get entries Tag data
	*EntrySize = Tag->EntrySize;									//	Set the returned values size
	*EntryType = Tag->EntryType;									//	... and type
	SpaceNeeded = *EntrySize + TagSize;
	Entry = malloc(SpaceNeeded);									//	Allocate the entry
	if(Entry == NULL)												//	Error, no memory
	{
		free(Tag);
		return NULL;
	}
	
	FRam_Read(Entry,Log->ActualPointer,SpaceNeeded);				//	Get complete data for entry
	Log->ActualPointer	= Tag->NextPointer;							//	Update the pointers
	Log->FreeSpace		+= SpaceNeeded;
	Log->EntryCount--;
	if(Log->EntryCount <= 1) Log->PrevPointer = NULL;
	if(Log->EntryCount == 0)
	{
		Log->NewestPointer = NULL;
		Log->ActualPointer = NULL;
	}
	WriteDataLog(Log);												//	Write back 
	free(Tag);														
	
	return Entry + TagSize;;										//	Return pointer to user's data 
}
//***********************************************************************************//

uint16 DataLogger_FreeLogEntry(void * Entry)						//	De-allocate entry buffer retrieved with DataLogger_GetLogEntry()
{
	if(Entry) free(Entry - TagSize);
	return Log->FreeSpace;	
}
//***********************************************************************************//

uint16 DataLogger_GetEntryCount(void)								//	Return count of actually logged records
{
	return Log->EntryCount;
	
}
//***********************************************************************************//

uint16 DataLogger_GetActualEntryType(void)							//	Return user supplied type of oldest record or NULL
{
struct LogTagging Tag;
	FRam_Read(&Tag,Log->ActualPointer,TagSize);
	
	return Tag.EntryType;
}
//***********************************************************************************//

uint16 DataLogger_GetActualEntrySize(void)							//	Return size of oldest record or NULL
{
struct LogTagging Tag;
	FRam_Read(&Tag,Log->ActualPointer,TagSize);
	
	return Tag.EntrySize;
}
//***********************************************************************************//
uint16 DataLogger_GetEntryType(void * Entry)						//	Return user supplied type of given entry or NULL
{
	return ((LogTagging)(Entry - TagSize))->EntryType;
}
//***********************************************************************************//

uint16 DataLogger_GetEntrySize(void * Entry)						//	Return size of given entry or NULL
{
	return ((LogTagging)(Entry - TagSize))->EntrySize;
}
//***********************************************************************************//

TimeDate DataLogger_GetTimeTag(void * Entry)						//	Return pointer to date and time structure of given entry or NULL
{
	return &((LogTagging)(Entry - TagSize))->LogTime;	
}
//***********************************************************************************//

void ReadCurrentTag(LogTagging Tag)									//	Real oldest tag into CurrentTag
{
	if(Log->ActualPointer == NULL)
	{
		Tag = NULL;
		return;
	}
	FRam_Read((uint8*)Tag,Log->ActualPointer,TagSize);
}
//***********************************************************************************//

void WriteDataLog(DataLog Log)										//	Write DataLog structure to FRam
{
		FRam_Write((uint8*)Log,(void *)FRamSpare,LogSize);			//	Store initialization in FRam
}
//***********************************************************************************//


