//***********************************************************************************//
//*	Project:	PSoC4 BLE Data Logging using the On-Board FRam						*//
//*																					*//
//*	File:		DataLogger.h														*//
//*	Version:	1.0																	*//
//*																					*//
//*	Author:		Bob Marlowe															*//
//*	(C) 2015	Jörg Meier Software - Entwicklung briefe@jmeiersoftware.de			*//
//***********************************************************************************//
//***********************************************************************************//
//*																					*//
//*		Principles of Operation														*//
//*																					*//
//*		Logging of data is done by maintaining a FIFO memory for datasets			*//
//*		of varying informations and varying sizes.									*//
//*		To handle the FIFO correctly some additional information are kept within	*//
//*		the FRam to continue logging after a power-down								*//
//*																					*//
//*		Logged records are maintained in form of a combination of a linked-list		*//
//*		and a circular buffer. This allows for inserting new records and retrieving	*//
//*		from beginning without having to move data down from the top when a record	*//
//*		is removed from the begin of the logging-area in the FRam					*//
//*																					*//
//*		The FRam is not used completely for logging, a part at the begin may be 	*//
//*		spared for other uses														*//
//*																					*//
//*																					*//
//***********************************************************************************//
#ifndef DataLogger_h
#define DataLogger_h
	
//***********************************************************************************//
//**************** Libraries ********************************************************//
//***********************************************************************************//
#include <project.h>
#include <TimeKeeper.h>

//***********************************************************************************//
//**************** Definitions and Macros *******************************************//
//***********************************************************************************//
#define	FRamSize			0x20000							//	1Mbit FM24V10 = 128 kByte
#define	FRamSpare			0x00804							//	Sparing 2KByte for other things
//*********** Do Not Change Beyond This Line ****************************************//
#define	LogOverhead			(sizeof(struct DataLog))		//	Bytes used for logging administration
#define MAGIC_NUMBER		0x0c0ffee1ul					//	Value of indicator for a valid initialized log (Magic Number)
//***********************************************************************************//
//******************* Structs, Typedefs and Enums ***********************************//
//***********************************************************************************//		

struct	LogTagging 											//	Tags automatically prepended to each logged record
		{
			uint16	EntryType;								//	User supplied record-type
			uint16	EntrySize;								//	Total number of user-bytes for this record
			void *	NextPointer;							//	Address of next record or NULL
			struct	TimeDate LogTime;						//	Time and date of log entry
		};
typedef struct LogTagging * LogTagging;						//	Define pointer to above struct

//***********************************************************************************//
//****************** Prototypes *****************************************************//
//***********************************************************************************//		
void DataLogger_Start(void);								//	Initialize and start the DataLogger
//***********************************************************************************//		
//*																					*//
//*		Data initialization and FRam components started. When the magic number 		*//
//*		stored in FRam does not match, FRam contents will get initialized which		*//
//*		means that all logged data will be overwritten.								*//
//*																					*//
//*																					*//
//***********************************************************************************//

//***********************************************************************************//		
void DataLogger_Stop(void);									//	Halt logging service, keep data
//***********************************************************************************//		
//*																					*//
//*		Stops all components that writes to FRam, so that no changes can be			*//
//*		made to the logged data																			*//
//*																					*//
//***********************************************************************************//

//***********************************************************************************//		
uint32 DataLogger_ClearLog(void);							//	Clear the log, return free space in bytes
//***********************************************************************************//		
//*																					*//
//*		All logged data will be overwritten, "Magic Number" written to FRam			*//
//*		Returns free space for logging																			*//
//*																					*//
//***********************************************************************************//

//***********************************************************************************//		
uint32 DataLogger_LogEntry(uint16 RecordType,void * Data,uint16 Count); // Store a log-record
//***********************************************************************************//		
//*																					*//
//*		Prepends the tag-data (Time, rec number..) to the record and writes			*//
//*		the data to the log.														*//
//*		RecordType:	User supplied record type to distinguish between different		*//
//*					log entries														*//
//*		Data:		Pointer to the buffer containing the log data to be written		*//
//*		Count:		Number of bytes to retrieve from data							*//
//*																					*//
//*		Returns:	Bytes remaining for logging										*//
//*																					*//
//*																					*//
//***********************************************************************************//

//***********************************************************************************//		
//uint16 DataLogger_GetRecord(uint16*RecordType,uint8*Buffer,uint16 BfSize);	//	Read oldest logged record, fill buffer and return free space in FRam
//***********************************************************************************//		
//*																					*//
//*		Retrieves the oldest logged record and deletes it from the log.				*//
//*		RecordType:	Returns the user-defined record-type for the log entry			*//
//*		Buffer:		Will be filled with the log entry								*//
//*		BfSize:		Size of above buffer. Nothing will be transferred when buffer	*//
//*					is not large enough to keep the complete record					*//
//*		Returns:	Bytes remaining for logging										*//
//*																					*//
//*																					*//
//*					What is with the additional tag - data???						*//
//*																					*//
//*																					*//
//***********************************************************************************//

//***********************************************************************************//		
void * DataLogger_GetLogEntry(uint16*RecordType,uint16 * EntrySize);	//	Read oldest logged record, allocate and fill buffer
//***********************************************************************************//		
//*																					*//
//*		Retrieves the oldest logged record and deletes it from the log.				*//
//*		RecordType:	Returns the user-defined record-type for the log entry			*//
//*		EntrySize:	Size of returned entry 											*//
//*		Returns:	Pointer to a buffer containig the data. Should be freed	using	*//
//*					DataLogger_FreeEntry() when not needed by the caller			*//
//*																					*//
//***********************************************************************************//

//***********************************************************************************//
uint16 DataLogger_FreeLogEntry(void * Entry);					//	De-allocate entry buffer retrieved with DataLogger_GetLogEntry()
//***********************************************************************************//
//*																					*//
//*		Frees the allocated buffer from DataLogger_GetLogEntry()					*//
//*		Entry:		Pointer to the entry received by calling DataLogger_GetLogEntry	*//
//*		Returns:	Number of bytes remaining for logging							*//
//*																					*//
//***********************************************************************************//

//***********************************************************************************//		
uint16 DataLogger_GetEntryCount(void);						//	Return count of actually logged records
//***********************************************************************************//		
//*																					*//
//*		Returns:	Total number of log entrie										*//
//*																					*//
//***********************************************************************************//

//***********************************************************************************//		
uint16 DataLogger_GetActualEntryType(void);					//	Return user supplied type of oldest record or NULL
//***********************************************************************************//		
//*																					*//
//*		Returns:	User supplied type of actual (first) log entry					*//
//*																					*//
//***********************************************************************************//

//***********************************************************************************//		
uint16 DataLogger_GetActualEntrySize(void);					//	Return size of oldest record or NULL
//***********************************************************************************//		
//*																					*//
//*		Returns:	Size of of actual (first) log entry in bytes					*//
//*																					*//
//***********************************************************************************//

//***********************************************************************************//		
uint16 DataLogger_GetEntryType(void * Entry);				//	Return user supplied type of given entry or NULL
//***********************************************************************************//		
//*																					*//
//*		Get the type of the user-data from the supplied entry						*//
//*		Entry:		Pointer to the entry received by calling DataLogger_GetLogEntry	*//
//*		Returns:	User supplied type of actual log entry							*//
//*																					*//
//***********************************************************************************//

//***********************************************************************************//		
uint16 DataLogger_GetEntrySize(void * Entry);				//	Return size of given entry or NULL
//***********************************************************************************//		
//*																					*//
//*		Get the size of the user-data from the supplied entry						*//
//*		Entry:		Pointer to the entry received by calling DataLogger_GetLogEntry	*//
//*		Returns:	Size of of actual (first) log entry in bytes					*//
//*																					*//
//***********************************************************************************//

//***********************************************************************************//		
TimeDate DataLogger_GetTimeTag(void * Entry);				//	Return pointer to date and time structure of given entry or NULL
//***********************************************************************************//		
//*																					*//
//*		Get the time and date from the supplied entry								*//
//*		Returns:	Pointer to a TimeDate structure									*//
//*																					*//
//***********************************************************************************//

#endif

/* [] END OF FILE */
