//***********************************************************************************//
//*	Project:	PSoC4 Maintaining a real-time clock (RTC)							*//
//*																					*//
//*	File:		TimeKeeper.c	 													*//
//*	Version:	1.0																	*//
//*																					*//
//*	Author:		Bob Marlowe															*//
//*	(C) 2014	Jörg Meier Software - Entwicklung briefe@jmeiersoftware.de			*//
//***********************************************************************************//
//***********************************************************************************//
//*																					*//
//*		Principles of Operation														*//
//*																					*//
//*		The Tick timer of the ARM CPU is used to maintain a real-time-clock			*//
//*		with an accuracy of 1 second. The precision depends on the oscillator		*//
//*		precision of the PSoC.														*//
//*																					*//
//***********************************************************************************//

	
//***********************************************************************************//
//**************** Libraries ********************************************************//
//***********************************************************************************//
#include <cytypes.h>
#include <TimeKeeper.h>
#include <project.h>

//***********************************************************************************//
//**************** Definitions and Macros *******************************************//
//***********************************************************************************//
#define SysTickIntNo	15										//	SysTick Interrupt Number

//***********************************************************************************//
//******************* Structs, Typedefs and Enums ***********************************//
//***********************************************************************************//
//	None yet

//***********************************************************************************//
//******************* Data Area *****************************************************//
//***********************************************************************************//
struct TimeDate ActualTimeDate;
struct Calendar ActualCalendarDate;
static uint16 DayInYear = 0;									//	Current day in current year
static uint16 LapDays	= 0;									//	Number of lapdays in current year is 0 or 1
static uint8 MonthDays[12] = {	31,28,31,30,31,30,				//	Number of days in each month
								31,31,30,31,30,31};
static const uint8 Tbl[12] = {  0,3,3,6,1,4,					//	Needed for calculating weekday
								6,2,5,0,3,5};
							
//***********************************************************************************//
//****************** Prototypes *****************************************************//
//***********************************************************************************//		
static void SysTicks(void);										//	Handler for System Ticks
static void InitializeTickCounter(void);						//	Start the 1Hz SysTick 
static void CalcLapDay(void);									//	Calculat lapday of current year

//***********************************************************************************//
//****************** Program Area ***************************************************//
//***********************************************************************************//

TimeDate TimeKeeper_Start(void)									//	Start the TimeKeeper, return address of TimerDate structure
{
	ActualTimeDate.DayOfYear	= 1;							//	Set date to January, 1st 2015
	ActualTimeDate.Year			= 2015;
	ActualTimeDate.Hour			= 0;
	ActualTimeDate.Minute		= 0;
	ActualTimeDate.Second		= 0;
	CalcLapDay();
	InitializeTickCounter();
	return &ActualTimeDate;	
}
//***********************************************************************************//

void TimeKeeper_Stop(void)										//	Stops the TimeKeeper
{
	TimeKeeper_Halt();
	CySysTickStop();
}
//***********************************************************************************//

void	TimeKeeper_SetTimeAndDate(TimeDate NewTimeDate)			//	Set current time and date
{
	TimeKeeper_Halt();
	memmove(&ActualTimeDate,NewTimeDate,sizeof(struct TimeDate));
	TimeKeeper_Run();
	CalcLapDay();
}
//***********************************************************************************//

void	TimeKeeper_GetTimeAndDate(TimeDate ActTime)				//	Get Time and date;
{
uint8 State;
	State = CyEnterCriticalSection();							//	No interrupts
	memmove(ActTime,&ActualTimeDate,sizeof(struct TimeDate));	//	Set destination
	CyExitCriticalSection(State);								//	Re-enable interrupts
}
//***********************************************************************************//

void	TimeKeeper_Halt(void)									//	Halts the TimeKeeper
{
	CySysTickDisableInterrupt();
}
//***********************************************************************************//

void	TimeKeeper_Run(void)									//	Resume halted TimeKeeper
{
	CySysTickEnableInterrupt();
}
//***********************************************************************************//

Calendar TimeKeeper_CalcDate(void)								//	Calculate Month, weekday etc from TimeDate
{
uint16	DayCounter = 0;	
uint8	Month = 0;
uint8	Nt, Nm, Njz, Njh, Nsj;									//	http://en.wikipedia.org/wiki/Determination_of_the_day_of_the_week german version
uint8	Jz, Jh;
	
	ActualCalendarDate.Year = ActualTimeDate.Year;			
	CalcLapDay();												// sets number of days in February
	while(ActualTimeDate.DayOfYear - DayCounter > MonthDays[Month])	
	{
		DayCounter += MonthDays[Month];	
	}
	ActualCalendarDate.Month = Month;
	ActualCalendarDate.DayInMonth	= ActualTimeDate.DayOfYear - DayCounter;
	Jh  = ActualTimeDate.Year /100;
	Jz	= ActualTimeDate.Year - (uint16)Jh * 100u;
	Nt  = Tbl[Month];
	Nm  = ActualCalendarDate.DayInMonth % 7;
	Njz = (Jz + (Jz / 4)) % 7;
	Njh = (3-(Jh % 4)) * 2;	
	if(LapDays && (Month <= 1))	Nsj = 6;
	else						Nsj = 0;
	
	ActualCalendarDate.WeekDay = (Nt + Nm + Njh + Njz + Nsj) % 7;
	
	return &ActualCalendarDate;	
}
//***********************************************************************************//

void InitializeTickCounter(void)
{
	CySysTickStart();
	CySysTickStop();
	CySysTickSetClockSource(CY_SYS_CLK_LFCLK_SRC_ILO);	//	Set clock to internal low-speed oscillator (ILO) 32kHz
	CySysTickSetReload(32768ul);						//	Period
	CySysTickSetCallback(4,SysTicks);
	CySysTickEnable();									//	Now calls SysTicks() each second
}	
//******************************************************************************//

void SysTicks(void)
{
/*														//	When a finer granuality is required...
static uint16 msCounter = 0;
	msCounter++;
	if(msCounter < 1000) return;
	msCounter = 0;
*/	
	ActualTimeDate.Second++;							//	One second is elapsed

	if(ActualTimeDate.Second > 59)						//	Second rollover
	{
		ActualTimeDate.Minute++;						//	Increment Minute 
		ActualTimeDate.Second = 0u;
	}

	if(ActualTimeDate.Minute > 59)						//	Minute rollover
	{ 
		ActualTimeDate.Hour++;
		ActualTimeDate.Minute = 0u;
	}
	
	if(ActualTimeDate.Hour > 23)						//	Hour rollover
	{
		DayInYear++;
		ActualTimeDate.Hour = 0;
	}
	
	if(DayInYear > 365 + LapDays)						//	Day in year rollover
	{
		ActualTimeDate.Year++;
		DayInYear = 1;
		CalcLapDay();									//	Calculate lapday for new year
	}
}
//******************************************************************************//

void CalcLapDay(void)
{
uint16 Result = 0;
uint16 Year = ActualTimeDate.Year;
	if(!(Year % 4))
	{
		Result = 1;
		if(!(Year % 100))
		{
			Result = 0;
			if(!(Year % 400))
			{
				Result = 1;								//	ActualTimeDate.Year is a lapyear
			}
		}
	}
	if(Result)	MonthDays[1] = 29;						//	Set days in February accordingly
	else		MonthDays[1] = 28;
	LapDays = Result;									//	Return 1 for lapyear, 0 when not
}
//******************************************************************************//



